#import "application.h"
#import "as_controller.h"
#include <Carbon/Carbon.h>

// ********************  Code from iGetKeys ******************* //
// © Copyright 2001 Apple Computer, Inc. All rights reserved.   //
// http://developer.apple.com/samplecode/iGetKeys/listing1.html //

typedef struct {
  short kchrID;
  Str255 KCHRname;
  short transtable[256];
} Ascii2KeyCodeTable;

OSStatus InitAscii2KeyCodeTable(Ascii2KeyCodeTable *ttable);
short AsciiToKeyCode(Ascii2KeyCodeTable *ttable, short asciiCode);
char KeyCodeToAscii(short virtualKeyCode);

enum {
  kTableCountOffset = 256+2,
  kFirstTableOffset = 256+4,
  kTableSize = 128
};

OSStatus InitAscii2KeyCodeTable(Ascii2KeyCodeTable *ttable) {
  unsigned char *theCurrentKCHR, *ithKeyTable;
  short count, i, j, resID;
  Handle theKCHRRsrc;
  ResType rType;
  for (i=0;i<256; i++) ttable->transtable[i] = -1;
  ttable->kchrID = (short) GetScriptVariable(smCurrentScript, smScriptKeys);
  theKCHRRsrc = GetResource('KCHR', ttable->kchrID);
  if (theKCHRRsrc == NULL) return resNotFound;
  GetResInfo(theKCHRRsrc,&resID,&rType,ttable->KCHRname);
  theCurrentKCHR = (unsigned char *)  (*theKCHRRsrc);
  count = * (short *) (theCurrentKCHR + kTableCountOffset);
  for (i=0; i<count; i++) {
	ithKeyTable = theCurrentKCHR + kFirstTableOffset + (i * kTableSize);
	for (j=0; j<kTableSize; j++) {
	  if ( ttable->transtable[ ithKeyTable[j] ] == -1)
		ttable->transtable[ ithKeyTable[j] ] = j;
	}
  }
  return noErr;
}

short AsciiToKeyCode(Ascii2KeyCodeTable *ttable, short asciiCode) {
  if (asciiCode >= 0 && asciiCode <= 255)
	return ttable->transtable[asciiCode];
  else return false;
}

char KeyCodeToAscii(short virtualKeyCode) {
  unsigned long state;
  long keyTrans;
  char charCode;
  Ptr kchr;
  state = 0;
  kchr = (Ptr) GetScriptVariable(smCurrentScript, smKCHRCache);
  keyTrans = KeyTranslate(kchr, virtualKeyCode, &state);
  charCode = keyTrans;
  if (!charCode) charCode = (keyTrans>>16);
  return charCode;
}

// ******************** END from iGetKeys ********************************* //


const UInt32 kMyHotKeyIdentifier='AShk';

EventHotKeyRef gMyHotKeyRef;
EventHotKeyID gMyHotKeyID;
EventHandlerUPP gAppHotKeyFunction;

pascal OSStatus MyHotKeyHandler(EventHandlerCallRef nextHandler,EventRef theEvent,void *userData) {
  NSLog(@"MyHotKeyHandler called...");
  as_controller *the_controller = [[[as_controller alloc] init] autorelease];
  //call an AppleScript (in this case, the hotkey_handler from the as_controller script):
  [the_controller exec_AS_handler:@"as_controller" the_handler:@"hotkey_handler" parameter_string:[NSString stringWithFormat:@"{nextHandler:%d, theEvent:%d, userData:%@}", nextHandler, theEvent, userData]]; 
  return true;
}

@implementation NSApplication (AS_HotKey)

- (void)registerMyHotkey:(short )the_key use_ctrl:(BOOL )control_down use_opt:(BOOL )option_down {
  NSLog(@"registerMyHotkey called...");

  Ascii2KeyCodeTable ttable;
  InitAscii2KeyCodeTable( &ttable );  
  const UInt32 MyHotKey = AsciiToKeyCode(&ttable, the_key);
  
  NSLog(@"MyHotKey AsciiToKeyCode = %u", MyHotKey);
  NSLog(@"MyHotKey KeyCodeToAscii = %c", KeyCodeToAscii(MyHotKey));
  
  EventTypeSpec eventType;
  gAppHotKeyFunction = NewEventHandlerUPP(MyHotKeyHandler);
  eventType.eventClass=kEventClassKeyboard;
  eventType.eventKind=kEventHotKeyPressed;
  InstallApplicationEventHandler(gAppHotKeyFunction,1,&eventType,NULL,NULL);
  gMyHotKeyID.signature=kMyHotKeyIdentifier;
  gMyHotKeyID.id=1;
  UInt32  modifiers;
  modifiers = cmdKey;
  if (option_down) {
	NSLog(@"Use option...");
	modifiers |= optionKey;
  }
  if (control_down) {
	NSLog(@"Use control...");
	modifiers |= controlKey;
  }
  
  RegisterEventHotKey(MyHotKey, modifiers, gMyHotKeyID, GetApplicationEventTarget(), 0, &gMyHotKeyRef);
  NSLog(@"registerMyHotkey done");
}

- (void)unregisterMyHotKey {
  NSLog(@"UnregisterEventHotKey called...");
  UnregisterEventHotKey(gMyHotKeyRef);
  NSLog(@"UnregisterEventHotKey done");
}
@end
